// this function verifies disk space in kilobytes
function verifyDiskSpace(dirPath, spaceRequired)
{
  var spaceAvailable;

  // Get the available disk space on the given path
  spaceAvailable = fileGetDiskSpaceAvailable(dirPath);

  // Convert the available disk space into kilobytes
  spaceAvailable = parseInt(spaceAvailable / 1024);

  // do the verification
  if(spaceAvailable < spaceRequired)
  {
    logComment("Insufficient disk space: " + dirPath);
    logComment("  required : " + spaceRequired + " K");
    logComment("  available: " + spaceAvailable + " K");
    return(false);
  }
  return(true);
}

var srDest = 449; // ???
var err;
var fProgram; // program folder
var sAppName    = "Nessi";
var sChromeName = "bci";
var sVersion    = "0.35";

err = initInstall(sAppName, sChromeName, sVersion); 
logComment("initInstall: " + err);

fProgram = getFolder("Program");
logComment("fProgram: " + fProgram);

if (verifyDiskSpace(fProgram, srDest))
{
  bciDir = getFolder("Chrome", sChromeName); // the subdir we want to install to
  err = addDirectory("", sChromeName, bciDir, "");
  logComment("addDirectory() returned: " + err);

  // xpi root path, xpi folder, local folder (object), local subfolder
  err = addDirectory("", "res", getFolder("Chrome", "../res"), "bci");
  logComment("addDirectory() returned: " + err);

  // components to add
  err = addFile("", "bci.dll", getFolder("Components"), "");
  err = addFile("", "bci.xpt", getFolder("Components"), "");
  logComment("added components: " + err);

  // files to add to chrome folder
  err = addFile("", "mozstuff/about.xhtml", getFolder("Chrome", "en-US/locale/en-US/global"), "");
  //err = addFile("", "mozstuff/about_de.xhtml", getFolder("Chrome", "de-AT/locale/de-AT/global"), "");
  logComment("added about files: " + err);

  // files to add to profile
  // try getFolder("Profile", "chrome") as suggested in NG
  err = addFile("", "mozstuff/userContent.css", getFolder("Preferences", "chrome"), "");
  err = addFile("", "mozstuff/userChrome.css", getFolder("Preferences", "chrome"), "");
  logComment("added user style sheets: " + err);
  // Bookmarks are a problem - file will be overwritten since browser is open
  // might be a blessing in disguise: don't overwrite user's existing bm's!
  //err = addFile("", "mozstuff/bookmarks.html", getFolder("Preferences"), "");
  //logComment("bms: " + err);
  err = addFile("", "mozstuff/user.js", getFolder("Preferences"), "");
  logComment("added user prefs: " + err);

  registerChrome(CONTENT | DELAYED_CHROME, getFolder(bciDir, "content/bci/"));
  registerChrome(LOCALE|DELAYED_CHROME, getFolder(bciDir,"locale/de-AT/bci/"));
  registerChrome(LOCALE|DELAYED_CHROME, getFolder(bciDir,"locale/en-US/bci/"));
  registerChrome(SKIN | DELAYED_CHROME, getFolder(bciDir,"skin/classic/bci/"));

  if (err == SUCCESS) {
    err = performInstall(); 
    logComment("performInstall() returned: " + err);
    alert(sAppName+" (v"+sVersion+") wurde erfolgreich installiert.\n"
          + "Bitte Nessi jetzt schliessen und neu starten!\n\n\n"
	  + sAppName+" (v"+sVersion+") has been successfully installed.\n"
          + "You must restart Nessi to complete the installation!\n"
	  );
  } else
    cancelInstall(err);
} else
cancelInstall(INSUFFICIENT_DISK_SPACE);
